using FitConnect.Models;
using FitConnect.Services;
using FitConnect.Services.Models;
using Microsoft.Extensions.Logging;

namespace FitConnect.BaseClasses;

public abstract class FunctionalBaseClass {
    protected readonly ILogger? Logger;
    // protected readonly FitConnectApiService ApiService;

    protected IOAuthService OAuthService;
    protected IRouteService RouteService;
    protected ISubmissionService SubmissionService;

    protected OAuthAccessToken? _oAuthToken;
    public bool IsAuthenticated => _oAuthToken is { IsValid: true };


    /// <summary>
    ///     Constructor for the FunctionalBaseClass
    /// </summary>
    /// <param name="environments">FitConnect endpoints</param>
    /// <param name="logger">ILogger implementation</param>
    /// <param name="oAuthService"></param>
    /// <param name="submissionService"></param>
    /// <param name="routeService"></param>
    /// <param name="destinationService"></param>
    /// <param name="certificate">The Encryption certificate</param>
    /// <example>
    ///     new Sender(logger, FitConnectEndpoints.Create(FitConnectEndpoints.EndpointType.Development))
    /// </example>
    protected FunctionalBaseClass(FitConnectEnvironment? environment,
        ILogger? logger,
        IOAuthService oAuthService,
        ISubmissionService submissionService,
        IRouteService routeService, IDestinationService destinationService) {
        DestinationService = destinationService;
        Environment = environment ??
                      FitConnectEnvironment.Development;

        Logger = logger;
        OAuthService = oAuthService;
        SubmissionService = submissionService;
        RouteService = routeService;
    }

    protected IDestinationService DestinationService { get; }


    internal Client Owner { get; set; }
    protected string? ClientId => Owner?.ClientId;
    protected string? ClientSecret => Owner?.ClientSecret;

    public FitConnectEnvironment Environment { get; }


    /// <summary>
    ///     Requesting the OAuth-Token
    /// </summary>
    /// <param name="clientId"></param>
    /// <param name="clientSecret"></param>
    /// <param name="scope"></param>
    /// <returns></returns>
    public async Task<OAuthAccessToken> AuthenticateAsync(string clientId, string clientSecret,
        string? scope = null) {
        _oAuthToken = await OAuthService.AuthenticateAsync(clientId, clientSecret, scope);
        return _oAuthToken;
    }

    /// <summary>
    ///     Receive the SET data
    /// </summary>
    /// <returns></returns>
    public Task<SecurityEventToken> GetSetDataAsync() {
        throw new NotImplementedException();
    }
}
