using System.Net;
using Autofac;
using FitConnect.Services;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Logging.Abstractions;

namespace FitConnect;

public static class DiContainer {
    private static IContainer? _container;
    public static IWebProxy? WebProxy { get; set; }

    public static void DisposeContainer() {
        _container?.Dispose();
        _container = null;
    }

    public static IContainer GetContainer(FitConnectEnvironment environment,
        ILogger? logger = null) {
        return _container ??=
            BuildContainer(environment, logger);
    }

    private static IContainer
        BuildContainer(FitConnectEnvironment environment,
            ILogger? logger = null) {
        var builder = new ContainerBuilder();

        if (logger == null)
            logger = NullLoggerFactory.Instance.CreateLogger("FitConnect");

        builder.RegisterInstance(environment).As<FitConnectEnvironment>();
        builder.RegisterInstance(logger).As<ILogger>();

        builder.RegisterType<Client>();
        builder.Register(c => new OAuthService(environment.TokenUrl)).As<IOAuthService>();
        builder.Register(c => new SubmissionService(environment.SubmissionUrl[0]))
            .As<ISubmissionService>();
        builder.Register(c => new DestinationService(environment.SubmissionUrl[0]))
            .As<IDestinationService>();
        builder.Register(c => new CasesService(environment.SubmissionUrl[0]))
            .As<ICasesService>();
        builder.Register(c => new RouteService(environment.RoutingUrl)).As<IRouteService>();

        builder.Register(c => new Sender(
            environment,
            new OAuthService(environment.TokenUrl),
            new SubmissionService(environment.SubmissionUrl[0]),
            new RouteService(environment.RoutingUrl),
            new DestinationService(environment.SubmissionUrl[0]),
            logger)).As<Sender>();

        builder.Register(c => new Subscriber(
            environment,
            new OAuthService(environment.TokenUrl),
            new SubmissionService(environment.SubmissionUrl[0]),
            new RouteService(environment.RoutingUrl),
            new DestinationService(environment.SubmissionUrl[0]),
            new CasesService(environment.SubmissionUrl[0]),
            logger)).As<Subscriber>();


        return builder.Build();
    }
}
